/* --------------------------------------------------------------------------
 * module_mixer.c
 * general code for the mixer module. This module doesn't control the
 * mixer device directly. It delegates all actions to the configured or
 * detected mixer module (OSS or ALSA) and keep only generic actions for
 * itself
 *
 * Copyright 2002-2004 Matthias Grimm
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 * -------------------------------------------------------------------------*/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include "pbbinput.h"

#include <pbb.h>

#include "gettext_macros.h"
#include "input_manager.h"
#include "module_mixer.h"
#include "module_ossmixer.h"
#include "module_alsamixer.h"
#include "support.h"

struct moddata_mixer {
	int soundsystem;     /* currently active sound system */
	int soundsystemcfg;  /* soundsystem to write to configfile */
} modbase_mixer;

int
mixer_init (struct tagitem *taglist)
{
	struct moddata_mixer *base = &modbase_mixer;
	char *names[] = {SOUNDSYSTEM_NONE_NAME, SOUNDSYSTEM_AUTO_NAME, SOUNDSYSTEM_OSS_NAME, SOUNDSYSTEM_ALSA_NAME};
	int rc;

	base->soundsystem    = SOUNDSYSTEM_NONE;
	base->soundsystemcfg = SOUNDSYSTEM_AUTO;

	if ((rc = mixer_handle_tags (MODE_CONFIG, taglist)))
		return rc;

	switch (base->soundsystemcfg) {
		case SOUNDSYSTEM_NONE:
			break;
		case SOUNDSYSTEM_AUTO: /* auto */
#if defined(WITH_ALSA) && defined(WITH_OSS)
			break;
#endif
		case SOUNDSYSTEM_ALSA: /* ALSA */
#ifdef WITH_ALSA
			base->soundsystem = SOUNDSYSTEM_ALSA;
			rc = alsamixer_init (taglist);
			break;
#endif
		case SOUNDSYSTEM_OSS: /* OSS */
#ifdef WITH_OSS
			base->soundsystem = SOUNDSYSTEM_OSS;
			rc = ossmixer_init (taglist);
			break;
#endif
		default:		
			break;
	}
	
	print_error (_("INFO: Currently active mixer: %s\n"), names[base->soundsystem]);
	
	if (base->soundsystem != SOUNDSYSTEM_NONE && rc == 0) {
		register_function (QUERYQUEUE, mixer_query);
		register_function (CONFIGQUEUE, mixer_configure);
	}
	return rc;
}

int
mixer_exit ()
{
	struct moddata_mixer *base = &modbase_mixer;
	
	switch (base->soundsystem) {
		case SOUNDSYSTEM_ALSA:
#ifdef WITH_ALSA
			alsamixer_exit();
			break;
#endif
		case SOUNDSYSTEM_OSS:
#ifdef WITH_OSS
			ossmixer_exit();
			break;
#endif
		default:
			break;
	}
	return 0;
}

void
mixer_query (struct tagitem *taglist)
{
	mixer_handle_tags (MODE_QUERY, taglist);
}

void
mixer_configure (struct tagitem *taglist)
{
	mixer_handle_tags (MODE_CONFIG, taglist);
}

int
mixer_handle_tags (int cfgure, struct tagitem *taglist)
{
	struct moddata_mixer *base = &modbase_mixer;

	while (taglist->tag != TAG_END) {
		switch (taglist->tag) {
		case TAG_SOUNDSYSTEM:
			if (cfgure)	tagerror (taglist, E_NOWRITE); 
			else		taglist->data = base->soundsystem;
			break;
		case TAG_SOUNDSYSTEMCFG:
			/* This tag configures the soundsystem. This change has to written to
			 * the configfile first and become active with next server start.
			 * Changing the soundsystem on the fly is not possible.
			 */
			if (cfgure)	base->soundsystemcfg = taglist->data;
			else		taglist->data = base->soundsystemcfg;
			break;
		}
		taglist++;
	}
	return 0;
}

